/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: GPELDeployWindow.java,v 1.15 2007/02/08 22:20:41 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya.gpel.gui;

import java.awt.event.ActionEvent;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JPanel;

import org.gpel.client.security.GpelUserX509Credential;
import org.ietf.jgss.GSSCredential;

import xsul5.MLogger;
import edu.indiana.extreme.xbaya.XBayaEngine;
import edu.indiana.extreme.xbaya.gpel.GPELClient;
import edu.indiana.extreme.xbaya.gpel.GPELException;
import edu.indiana.extreme.xbaya.gpel.script.BPELScript;
import edu.indiana.extreme.xbaya.gui.ErrorMessages;
import edu.indiana.extreme.xbaya.gui.GridPanel;
import edu.indiana.extreme.xbaya.gui.XBayaDialog;
import edu.indiana.extreme.xbaya.gui.XBayaLabel;
import edu.indiana.extreme.xbaya.gui.XBayaTextArea;
import edu.indiana.extreme.xbaya.gui.XBayaTextField;
import edu.indiana.extreme.xbaya.myproxy.MyProxyClient;
import edu.indiana.extreme.xbaya.myproxy.gui.MyProxyChecker;
import edu.indiana.extreme.xbaya.security.XBayaSecurity;
import edu.indiana.extreme.xbaya.wf.Workflow;

/**
 * @author Satoshi Shirasuna
 */
public class GPELDeployWindow {

    private static final MLogger logger = MLogger.getLogger();

    protected XBayaEngine engine;

    private MyProxyChecker myProxyChecker;

    protected XBayaDialog dialog;

    private JButton newButton;

    private JButton redeployButton;

    protected XBayaTextField nameTextField;

    protected XBayaTextArea descriptionTextArea;

    private GPELDeployer deployer;

    private Workflow workflow;

    /**
     * Constructs a GPELDeployWindow.
     * 
     * @param engine
     */
    public GPELDeployWindow(XBayaEngine engine) {
        this.engine = engine;
        this.deployer = new GPELDeployer(engine);
        this.myProxyChecker = new MyProxyChecker(this.engine);
        initGUI();
    }

    /**
     * Shows the window.
     */
    public void show() {
        GPELClient gpelClient = this.engine.getGPELClient();
        if (gpelClient.isSecure()) {
            // Check if the proxy is loaded.
            boolean loaded = this.myProxyChecker.loadIfNecessary();
            if (!loaded) {
                return;
            }
            // Creates a secure channel in gpel.
            MyProxyClient myProxyClient = this.engine.getMyProxyClient();
            GSSCredential proxy = myProxyClient.getProxy();
            GpelUserX509Credential credential = new GpelUserX509Credential(
                    proxy, XBayaSecurity.getTrustedCertificates());
            try {
                gpelClient.setGpelUserX509Credential(credential);
            } catch (GPELException e) {
                this.engine.getErrorWindow().error(ErrorMessages.GPEL_ERROR, e);
                return;
            }
        }

        this.workflow = this.engine.getWorkflow();
        BPELScript bpel = new BPELScript(this.workflow);

        // Check if there is any errors in the workflow first.
        ArrayList<String> warnings = new ArrayList<String>();
        if (!bpel.validate(warnings)) {
            StringBuilder buf = new StringBuilder();
            for (String warning : warnings) {
                buf.append("- ");
                buf.append(warning);
                buf.append("\n");
            }
            this.engine.getErrorWindow().warning(buf.toString());
            return;
        }

        String name = this.workflow.getName();
        String description = this.workflow.getDescription();

        this.nameTextField.setText(name);
        this.descriptionTextArea.setText(description);

        if (this.workflow.getGPELTemplateID() == null) {
            this.redeployButton.setEnabled(false);
        } else {
            this.redeployButton.setEnabled(true);
        }

        logger.finest("before show");
        this.dialog.show();
    }

    /**
     * Hides the window.
     */
    public void hide() {
        this.dialog.hide();
    }

    protected void deploy(boolean redeploy) {
        // Set the name and description to the graph.
        String name = this.nameTextField.getText();
        String description = this.descriptionTextArea.getText();

        if (name.trim().length() == 0) {
            this.engine.getErrorWindow().warning(
                    "You need to set the name of the workflow.");
        } else {
            // Use this method in order to change the name of the tab.
            this.engine.getGUI().getGraphCanvas().setNameAndDescription(name,
                    description);
            hide();
            this.deployer.deploy(redeploy);
        }
    }

    /**
     * Initializes the GUI
     */
    private void initGUI() {
        this.nameTextField = new XBayaTextField();
        XBayaLabel nameLabel = new XBayaLabel("Name", this.nameTextField);

        this.descriptionTextArea = new XBayaTextArea();
        XBayaLabel descriptionLabel = new XBayaLabel("Description",
                this.descriptionTextArea);

        GridPanel mainPanel = new GridPanel();
        mainPanel.add(nameLabel);
        mainPanel.add(this.nameTextField);
        mainPanel.add(descriptionLabel);
        mainPanel.add(this.descriptionTextArea);
        mainPanel.layout(2, 2, 1, 1);

        this.newButton = new JButton("Deploy New");
        this.newButton.setDefaultCapable(true);
        this.newButton.addActionListener(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                deploy(false);
            }
        });

        this.redeployButton = new JButton("Redeploy");
        this.redeployButton.setDefaultCapable(true);
        this.redeployButton.addActionListener(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                deploy(true);
            }
        });

        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                hide();
            }
        });

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(this.newButton);
        buttonPanel.add(this.redeployButton);
        buttonPanel.add(cancelButton);

        this.dialog = new XBayaDialog(this.engine,
                "Deploy the Workflow to the BPEL Engine", mainPanel,
                buttonPanel);
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
